// Libraries
import t from 'typy'
import PropTypes from 'prop-types'
import classNames from 'classnames'
import React, { Component } from 'react'

import { injectStripe, CardElement } from 'react-stripe-elements'

class StripeFields extends Component {
  constructor () {
    super()

    this.state = {
      errors: {
        CardElement: ''
      },
      dataAvailable: {
        CardElement: false
      }
    }
  }

  componentDidUpdate (prevProps) {
    if (!prevProps.cardTokenRequested && this.props.cardTokenRequested) {
      const billingAddress = this.props.billingAddress

      this.props.stripe.createPaymentMethod('card',
        {
          billing_details: {
            name: `${billingAddress.first_name} ${billingAddress.last_name}`,
            email: t(billingAddress, 'meta_attributes.email.value').safeString,
            phone: t(billingAddress, 'meta_attributes.phone_number.value').safeString,
            address: {
              city: billingAddress.city,
              country: billingAddress.country,
              line1: billingAddress.address_line_1,
              line2: billingAddress.address_line_2,
              state: billingAddress.state,
              postal_code: billingAddress.postcode
            }
          }
        }
      ).then(({ paymentMethod, error }) => {
        this.props.onPaymentMethodCreate(error ? { error } : { paymentMethod })
      })
    }

    if (!prevProps.paymentIntent.requires_action && this.props.paymentIntent.requires_action) {
      this.handleAction()
    }
  }

  handleAction () {
    this.props.stripe.handleCardAction(this.props.paymentIntent.payment_intent_client_secret).then((response) => {
      if (response.error) {
        // Show error in payment form
        this.props.setCardErrors(true)
        this.props.onPaymentMethodCreate({ error: { message: response.error.message } })
      } else {
        // The card action has been handled,
        // the PaymentIntent can be confirmed again on the server
        this.props.confirmPaymentIntent(response.paymentIntent.id)
      }
    })
  }

  handleChange (fieldName, e) {
    let errorMessage = ''

    if (e.error || e.complete === false) {
      errorMessage = e.error ? e.error.message : 'Incomplete card details'
    }

    this.setState({
      errors: Object.assign(this.state.errors, { [fieldName]: errorMessage }),
      dataAvailable: Object.assign(this.state.dataAvailable, { [fieldName]: !e.empty })
    })

    e.complete ? this.props.setStripeFormComplete(true) : this.props.setStripeFormComplete(false)

    this.checkDataValidity()
  }

  checkDataValidity () {
    const errors = this.state.errors
    const dataAvailable = this.state.dataAvailable
    const error = Object.keys(errors).every((key) => errors[key] === '') === true
    const allFields = Object.keys(dataAvailable).every((key) => dataAvailable[key] === true) === true

    this.props.setCardErrors(!(error && allFields))
  }

  renderValidationMessageFor (fieldName) {
    let errorMessage = this.state.errors[fieldName]

    return (
      errorMessage && <div className='o-form__input-field-error'>{ errorMessage }</div>
    )
  }

  render () {
    let { errors } = this.state

    return <>
      <label>
        Please enter your credit card details *
        <CardElement
          className={classNames('o-form__input-field', { 'o-form__input-field-validation': (errors.CardElement !== '') })}
          onChange={this.handleChange.bind(this, 'CardElement')}
          hidePostalCode={true}
        />
      </label>

      { this.renderValidationMessageFor('CardElement') }
    </>
  }
}

StripeFields.propTypes = {
  billingAddress: PropTypes.object,
  cardTokenRequested: PropTypes.bool,
  onPaymentMethodCreate: PropTypes.func,
  setCardErrors: PropTypes.func,
  stripe: PropTypes.object
}

export default injectStripe(StripeFields)
